"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const zod_1 = require("zod");
const utils_1 = require("../utils");
const error_1 = require("../utils/error");
const http_1 = require("../utils/http");
const schema_1 = require("./schema");
exports.default = ({ strapi }) => ({
    update(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const { workflowId } = ctx.params;
                const { assigneeId, assigneeType } = ctx.request.body;
                yield Promise.all([
                    yield (0, utils_1.validateRequest)(Number(workflowId), schema_1.workflowPathParamsSchema),
                    yield (0, utils_1.validateRequest)(ctx.request.body, schema_1.assignBodySchema)
                ]);
                const entry = yield (0, utils_1.findEntryById)(workflowId);
                if (!entry) {
                    throw new error_1.NotFoundError("No Entry found!");
                }
                const isEntryApprover = `${assigneeType}`.toLowerCase().trim() === "user" && (entry === null || entry === void 0 ? void 0 : entry.approverAsUser);
                const user = yield (0, utils_1.isUserEligible)(assigneeId);
                const isUserSameAsApprover = isEntryApprover && Number(isEntryApprover.id) === Number(assigneeId);
                const isAssigneeSuperAdmin = isEntryApprover && (0, utils_1.isSuperAdmin)(user === null || user === void 0 ? void 0 : user.roles);
                if (isUserSameAsApprover && !isAssigneeSuperAdmin) {
                    throw new error_1.ForbiddenError("Assignee CAN'T be the same as APPROVER.");
                }
                const assigneeRoleEligbible = `${assigneeType}`.toLowerCase().trim() === "role" && (yield (0, utils_1.isRoleEligible)(assigneeId));
                const userEligible = `${assigneeType}`.toLowerCase().trim() === "user" && user;
                if (`${assigneeType}`.toLowerCase().trim() === "role" && !assigneeRoleEligbible) {
                    throw new error_1.ForbiddenError("Role Not Eligible");
                }
                if (`${assigneeType}`.toLowerCase().trim() === "user" && !userEligible) {
                    throw new error_1.ForbiddenError("User Not Eligible");
                }
                ctx.status = 201;
                const assignee = yield strapi
                    .plugin("review-workflow")
                    .service("assigneeService")
                    .update({
                    assigneeId,
                    workflowId,
                    assigneeType: (0, utils_1.capitalizeFirstLetter)(assigneeType),
                    entry
                });
                ctx.body = new http_1.SuccessResponseObject("Assignee updated", assignee);
                return ctx;
            }
            catch (err) {
                let status = 500;
                let message = "some error occurred"; // Don't expose db error and other errors internal to code like undefined etc.
                let data;
                if (err instanceof error_1.CustomError) {
                    status = err.statusCode;
                    message = err.message;
                }
                if (err instanceof zod_1.ZodError) {
                    status = 400;
                    message = err.name;
                    data = JSON.parse(err.message);
                }
                yield strapi.plugin("sentry").service("sentry").sendError(JSON.stringify(err));
                ctx.status = status;
                ctx.body = new http_1.ErrorResponseObject(`${message}`, data);
                return ctx;
            }
        });
    }
});
