"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const error_1 = require("../utils/error");
const http_1 = require("../utils/http");
const schema_1 = require("./schema");
exports.default = ({ strapi }) => ({
    allStages(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const stages = yield strapi.plugin("review-workflow").service("workflowService").allStages();
                ctx.status = 200;
                ctx.body = new http_1.SuccessResponseObject("Successfully fetch all stages", stages);
            }
            catch (err) {
                const errResponse = yield (0, utils_1.handleErrors)(err);
                ctx.status = errResponse.status;
                ctx.body = errResponse.body;
                return ctx;
            }
        });
    },
    create(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield (0, utils_1.validateRequest)(ctx.request.body, schema_1.requestBodySchema);
                const { taskId, type } = ctx.request.body;
                const entryExist = yield (0, utils_1.findOneEntry)({ id: taskId, uid: type });
                if (Object.keys(entryExist).length !== 0) {
                    throw new error_1.ConflictError("Duplicate not allowed.");
                }
                const createWorkflow = yield strapi
                    .plugin("review-workflow")
                    .service("workflowService")
                    .create(ctx.request.body, ctx.state.user);
                ctx.status = 201;
                ctx.body = new http_1.SuccessResponseObject("Workflow successfully created", createWorkflow);
                return ctx;
            }
            catch (err) {
                const errResponse = yield (0, utils_1.handleErrors)(err);
                ctx.status = errResponse.status;
                ctx.body = errResponse.body;
                return ctx;
            }
        });
    },
    update(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const data = ctx.request.body;
                const user = ctx.state.user;
                const { id } = ctx.params;
                yield Promise.all([
                    (0, utils_1.validateRequest)(Number(id), schema_1.idPathParams),
                    (0, utils_1.validateRequest)(data, schema_1.updateRequestBodySchema)
                ]);
                const entry = yield (0, utils_1.findEntryById)(id);
                if (!entry) {
                    throw new error_1.NotFoundError("No entry found");
                }
                yield (0, utils_1.checkApproverAndAssignee)(entry, user, data);
                const updateWorkflow = yield strapi
                    .plugin("review-workflow")
                    .service("workflowService")
                    .updateEntry({ id, data, user, entry });
                ctx.status = 200;
                ctx.body = new http_1.SuccessResponseObject("Workflow updated", updateWorkflow);
                return ctx;
            }
            catch (err) {
                const errResponse = yield (0, utils_1.handleErrors)(err);
                ctx.status = errResponse.status;
                ctx.body = errResponse.body;
                return ctx;
            }
        });
    },
    delete(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const { id } = ctx.params;
                const deletWokflow = yield strapi
                    .plugin("review-workflow")
                    .service("workflowService")
                    .deleteEntry({ id });
                ctx.status = 204;
                ctx.body = new http_1.SuccessResponseObject("Workflow deleted", deletWokflow);
                return ctx;
            }
            catch (err) {
                const errResponse = yield (0, utils_1.handleErrors)(err);
                ctx.status = errResponse.status;
                ctx.body = errResponse.body;
                return ctx;
            }
        });
    },
    eligibleUsers(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const eligibleUsers = yield strapi
                    .plugin("review-workflow")
                    .service("workflowService")
                    .users();
                ctx.status = 200;
                ctx.body = new http_1.SuccessResponseObject("Eligible users fetched", eligibleUsers);
                return ctx;
            }
            catch (err) {
                const errResponse = yield (0, utils_1.handleErrors)(err);
                ctx.status = errResponse.status;
                ctx.body = errResponse.body;
                return ctx;
            }
        });
    },
    eligibleRoles(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const eligibleRoles = yield strapi
                    .plugin("review-workflow")
                    .service("workflowService")
                    .roles();
                ctx.status = 200;
                ctx.body = new http_1.SuccessResponseObject("Eligible Roles fetched", eligibleRoles);
                return ctx;
            }
            catch (err) {
                const errResponse = yield (0, utils_1.handleErrors)(err);
                ctx.status = errResponse.status;
                ctx.body = errResponse.body;
                return ctx;
            }
        });
    },
    getSingle(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const { target_id, target_type } = ctx.params;
                ctx.params.target_id = Number(ctx.params.target_id);
                yield (0, utils_1.validateRequest)(ctx.params, schema_1.singleEntrySchema);
                const entry = yield (0, utils_1.findOneEntry)({ uid: target_type, id: target_id });
                ctx.status = 200;
                ctx.body = new http_1.SuccessResponseObject("Workflow successfully fetched", entry);
                return ctx;
            }
            catch (err) {
                const errResponse = yield (0, utils_1.handleErrors)(err);
                ctx.status = errResponse.status;
                ctx.body = errResponse.body;
                return ctx;
            }
        });
    }
});
