"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-restricted-imports */
const error_1 = require("../utils/error");
const check_if_can_publish_1 = require("./check-if-can-publish");
const delete_entity_1 = require("./delete-entity");
exports.default = () => {
    return (ctx, next) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const url = ctx.request.url;
            const splitUrl = url.split("/");
            const isContentManager = splitUrl[1] === "content-manager";
            if (!isContentManager) {
                return yield next();
            }
            const isActionPublish = isContentManager &&
                splitUrl[splitUrl.length - 2] === "actions" &&
                splitUrl[splitUrl.length - 1] === "publish";
            const isActionDelete = isContentManager && ctx.request.method === "DELETE";
            if (isActionPublish) {
                const isCollectionTypes = splitUrl[2] === "collection-types";
                const slug = splitUrl[3];
                const collectionId = isCollectionTypes ? splitUrl[4] : "";
                yield (0, check_if_can_publish_1.isPublishAllowed)({ uid: slug, id: collectionId });
                return yield next();
            }
            if (isActionDelete) {
                const isCollectionTypes = splitUrl[2] === "collection-types";
                const slug = splitUrl[3];
                const collectionId = isCollectionTypes ? splitUrl[4] : "";
                yield (0, delete_entity_1.deleteEntityOnWorkflowTable)({ uid: slug, id: collectionId });
                return yield next();
            }
            return yield next();
        }
        catch (error) {
            let status = 500;
            let message = "some error occurred"; // Don't expose db error and other errors internal to code like undefined etc.
            if (error instanceof error_1.CustomError) {
                status = error.statusCode;
                message = error.message;
            }
            ctx.throw(status, `${message}`);
            return;
        }
    });
};
