"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const error_1 = require("../utils/error");
var STAGES;
(function (STAGES) {
    STAGES["approved"] = "Approved";
    STAGES["rejected"] = "Rejected";
    STAGES["hold"] = "Hold";
    STAGES["in_progress"] = "In progress";
    STAGES["ready_for_review"] = "Ready for review";
})(STAGES || (STAGES = {}));
exports.default = ({ strapi }) => ({
    allStages() {
        return __awaiter(this, void 0, void 0, function* () {
            return Object.values(STAGES);
        });
    },
    users() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const eligibleUsers = (yield ((_a = strapi.entityService) === null || _a === void 0 ? void 0 : _a.findMany("admin::user", {
                fields: ["id", "firstname", "lastname", "username", "email"],
                filters: { blocked: false, isActive: true }
            })));
            const updatedArray = eligibleUsers.map((item) => (Object.assign(Object.assign({}, item), { name: `${item.firstname} ${item === null || item === void 0 ? void 0 : item.lastname}` })));
            return updatedArray;
        });
    },
    roles() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const eligibleRoles = yield ((_a = strapi.entityService) === null || _a === void 0 ? void 0 : _a.findMany("admin::role", {
                fields: ["id", "name", "code"]
            }));
            return eligibleRoles;
        });
    },
    updateEntry(params) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const { id, data, user } = params;
            if (data.taskId) {
                data.id = data.taskId;
                delete data.taskId;
            }
            return yield ((_a = strapi.entityService) === null || _a === void 0 ? void 0 : _a.update("plugin::review-workflow.workflow", id, {
                data: Object.assign(Object.assign({}, data), { updated_by_id: user.id })
            }));
        });
    },
    deleteEntry(id) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            return (_a = strapi.entityService) === null || _a === void 0 ? void 0 : _a.delete("plugin::review-workflow.workflow", id);
        });
    },
    create(input, user) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const data = mapObject(input, user);
            const stage = `${data.stage}`.trim().toLowerCase();
            if (stage === "approved") {
                throw new error_1.BadRequestError("You can't transition to 'Approved' immediately");
            }
            const response = yield ((_a = strapi.entityService) === null || _a === void 0 ? void 0 : _a.create("plugin::review-workflow.workflow", {
                data
            }));
            return response;
        });
    }
});
const mapObject = (obj, user) => {
    return {
        stage: obj.stage,
        related: {
            id: obj.taskId,
            __type: obj.type
        },
        created_by_id: user.id
    };
};
