"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleErrors = exports.checkApproverAndAssignee = exports.capitalizeFirstLetter = exports.unlinkRelation = exports.mapRoleRelationsData = exports.mapUserRelationsData = exports.mapUserData = exports.stageApproved = exports.isRoleEligible = exports.isUserEligible = exports.isApproveSameAsUser = exports.isSuperAdmin = exports.validateRequest = exports.findEntryById = exports.findOneEntry = void 0;
const zod_1 = require("zod");
const error_1 = require("./error");
const http_1 = require("./http");
const findOneEntry = (params) => __awaiter(void 0, void 0, void 0, function* () {
    const { uid, id } = params;
    if (uid && id) {
        const entry = yield strapi.db.query("plugin::review-workflow.workflow").findOne({
            where: { target_id: id, target_type: uid },
            populate: ["approverAsUser", "approverAsRole", "assigneeAsUser", "assigneeAsRole"]
        });
        const response = Object.assign({}, entry);
        let normalizeAssignee;
        let normalizeApprover;
        if (entry === null || entry === void 0 ? void 0 : entry.approverAsUser) {
            normalizeApprover = (0, exports.mapUserRelationsData)(entry.approverAsUser);
            response.approverAsUser = normalizeApprover;
        }
        if (entry === null || entry === void 0 ? void 0 : entry.assigneeAsUser) {
            normalizeAssignee = (0, exports.mapUserRelationsData)(entry.assigneeAsUser);
            response.assigneeAsUser = normalizeAssignee;
        }
        return response !== null && response !== void 0 ? response : null;
    }
    const entry = yield strapi.db.query("plugin::review-workflow.workflow").findOne({
        where: { target_type: uid },
        populate: ["approverAsUser", "approverAsRole", "assigneeAsUser", "assigneeAsRole"]
    });
    const response = Object.assign({}, entry);
    if (response === null || response === void 0 ? void 0 : response.approverAsUser) {
        response.approverAsUser = (0, exports.mapUserRelationsData)(entry.approverAsUser);
    }
    if (response === null || response === void 0 ? void 0 : response.assigneeAsUser) {
        response.assigneeAsUser = (0, exports.mapUserRelationsData)(entry.assigneeAsUser);
    }
    return response !== null && response !== void 0 ? response : null;
});
exports.findOneEntry = findOneEntry;
const findEntryById = (id) => __awaiter(void 0, void 0, void 0, function* () {
    return yield strapi.entityService.findOne("plugin::review-workflow.workflow", id, {
        populate: {
            assigneeAsUser: {
                fields: ["id", "firstname", "lastname", "username", "email"]
            },
            assigneeAsRole: {
                fields: ["id", "name", "code"]
            },
            approverAsUser: {
                fields: ["id", "firstname", "lastname", "username", "email"]
            },
            approverAsRole: {
                fields: ["id", "name", "code"]
            }
        }
    });
});
exports.findEntryById = findEntryById;
const validateRequest = (payload, requestBodySchema) => __awaiter(void 0, void 0, void 0, function* () {
    return yield requestBodySchema.parse(payload);
});
exports.validateRequest = validateRequest;
const isSuperAdmin = (role) => {
    const IsUserSuperAdmin = role.filter((role) => `${role.name}`.trim().toLowerCase() === `super admin`.trim());
    return IsUserSuperAdmin.length ? true : false;
};
exports.isSuperAdmin = isSuperAdmin;
const isApproveSameAsUser = (approverId, userId) => {
    return approverId === userId;
};
exports.isApproveSameAsUser = isApproveSameAsUser;
const isUserEligible = (userId) => __awaiter(void 0, void 0, void 0, function* () {
    const response = yield strapi.entityService.findOne("admin::user", userId, {
        fields: ["id", "firstname", "lastname", "username", "email"],
        filters: { blocked: false, isActive: true },
        populate: ['roles']
    });
    return response;
});
exports.isUserEligible = isUserEligible;
const isRoleEligible = (id) => __awaiter(void 0, void 0, void 0, function* () {
    const role = yield strapi.entityService.findOne("admin::role", id, {
        fields: ["id", "name", "code"]
    });
    return role;
});
exports.isRoleEligible = isRoleEligible;
const stageApproved = (stage) => `${stage}`.trim().toLowerCase() === "approved";
exports.stageApproved = stageApproved;
const mapUserData = (obj) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    return {
        id: obj.id,
        firstname: (_b = (_a = obj.approver) === null || _a === void 0 ? void 0 : _a.firstname) !== null && _b !== void 0 ? _b : (_c = obj.assignee) === null || _c === void 0 ? void 0 : _c.firstname,
        lastname: (_e = (_d = obj.approver) === null || _d === void 0 ? void 0 : _d.lastname) !== null && _e !== void 0 ? _e : (_f = obj.assignee) === null || _f === void 0 ? void 0 : _f.lastname,
        email: (_h = (_g = obj.approver) === null || _g === void 0 ? void 0 : _g.email) !== null && _h !== void 0 ? _h : (_j = obj.assignee) === null || _j === void 0 ? void 0 : _j.email
    };
};
exports.mapUserData = mapUserData;
const mapUserRelationsData = (obj) => {
    return {
        id: obj.id,
        name: `${obj.firstname} ${obj === null || obj === void 0 ? void 0 : obj.lastname}`,
        firstname: obj.firstname,
        lastname: obj === null || obj === void 0 ? void 0 : obj.lastname,
        email: obj.email
    };
};
exports.mapUserRelationsData = mapUserRelationsData;
const mapRoleRelationsData = (obj) => {
    return {
        id: obj.id,
        name: obj.name,
        code: obj.code
    };
};
exports.mapRoleRelationsData = mapRoleRelationsData;
const unlinkRelation = (data) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const { id, link, type } = data;
    const unlink = yield ((_a = strapi.entityService) === null || _a === void 0 ? void 0 : _a.update("plugin::review-workflow.workflow", id, {
        data: {
            [`${link}`]: null,
            [`${type}`]: null
        },
        populate: [`${link}`]
    }));
    return unlink ? true : false;
});
exports.unlinkRelation = unlinkRelation;
const capitalizeFirstLetter = (str) => {
    return str.charAt(0).toUpperCase() + str.slice(1);
};
exports.capitalizeFirstLetter = capitalizeFirstLetter;
const checkApproverAndAssignee = (entry, user, data) => __awaiter(void 0, void 0, void 0, function* () {
    const { approverAsRole, approverAsUser, assigneeAsUser, assigneeAsRole, approverType } = entry;
    const approver = approverAsRole !== null && approverAsRole !== void 0 ? approverAsRole : approverAsUser;
    const assignee = assigneeAsUser !== null && assigneeAsUser !== void 0 ? assigneeAsUser : assigneeAsRole;
    const isUserSuperAdmin = (0, exports.isSuperAdmin)(user.roles);
    const isStageApproved = (data === null || data === void 0 ? void 0 : data.stage) && (0, exports.stageApproved)(data.stage);
    if (isStageApproved && !(approver || assignee)) {
        throw new error_1.ForbiddenError("Assignee and Approver required.");
    }
    const userHasRolePermission = (entry === null || entry === void 0 ? void 0 : entry.approverAsRole) &&
        user.roles.filter((role) => `${role.name}`.toLowerCase().trim() === `${entry.approverAsRole.name}`.toLowerCase().trim());
    if (approverType === "role" &&
        isStageApproved &&
        !isUserSuperAdmin &&
        !userHasRolePermission.length) {
        throw new error_1.ForbiddenError("Don't have permission to approve this task.");
    }
    if (approverType === "user" &&
        !isUserSuperAdmin &&
        isStageApproved &&
        entry.approverAsUser.id !== user.id) {
        throw new error_1.ForbiddenError("Don't have permission to approve this task.");
    }
});
exports.checkApproverAndAssignee = checkApproverAndAssignee;
const handleErrors = (err) => __awaiter(void 0, void 0, void 0, function* () {
    let status = 500;
    let message = "some error occurred"; // Don't expose db error and other errors internal to code like undefined etc.
    let data;
    if (err instanceof error_1.CustomError) {
        status = err.statusCode;
        message = err.message;
    }
    if (err instanceof zod_1.ZodError) {
        status = 400;
        message = err.name;
        data = JSON.parse(err.message);
    }
    if (process.env.NODE_ENV !== "test") {
        yield strapi.plugin("sentry").service("sentry").sendError(JSON.stringify(err));
    }
    const body = new http_1.ErrorResponseObject(`${message}`, data);
    return { status, body };
});
exports.handleErrors = handleErrors;
